
/*****************************************************************************
** Internal: FinishInstall()
*/

void FinishInstall(struct Install *Self)
{
   /*** Disable the installation buttons ***/

   if (Self->NextButtonID) ActionMsg(AC_Disable, Self->NextButtonID, NULL);
   if (Self->PrevButtonID) ActionMsg(AC_Disable, Self->PrevButtonID, NULL);
   if (Self->CancelButtonID) ActionMsg(AC_Disable, Self->CancelButtonID, NULL);

   /*** Print the completion text ***/

   DisplayXML(Self, "<item><b><font colour=\"#202080\">Install Complete</font></b></item><item/><item>Installation complete.  You may now quit the installer, or choose another program to install.</item>");

   /*** Reenable the navigation area ***/

   if (Self->NavigationID) ActionMsg(AC_Enable, Self->NavigationID, NULL);

   /*** Exit ***/

   if (Self->prvUninstall) {
      if (Self->prvPackageName) {
         struct mtGetXMLString getxml;
         struct acWrite write;
         UBYTE strTail[] = "</uninstall>\n";
         LONG len, i;

         DPrintF("Install:","Completing the uninstall file.");

         if (Self->prvUninstallIndex) {
            DPrintF("Install:","Writing custom uninstall tags to end of XML file.");

            getxml.Index = Self->prvUninstallIndex;
            getxml.Flags = XMF_INCLUDESIBLINGS;
            if (Action(MT_GetXMLString, Self->prvXML, &getxml) IS ERR_Okay) {
               for (i=0; getxml.Result[i]; i++);
               write.Buffer = getxml.Result;
               write.Length = i;
               Action(AC_Write, Self->prvUninstall, &write);

               write.Buffer = "\n";
               write.Length = 1;
               Action(AC_Write, Self->prvUninstall, &write);

               FreeMemory(getxml.Result);
            }
            else DPrintF("@Install:","GetXMLString() failed.");
         }

         /*** Write out the </uninstall tag at the end of the file ***/

         write.Buffer = strTail;
         write.Length = sizeof(strTail) - 1;
         Action(AC_Write, Self->prvUninstall, &write);

         /*** Move the uninstall file from its temporary location to the destination ***/

         for (len=0; Self->prvPackageName[len]; len++);

         {
            UBYTE buffer[sizeof("system:config/uninstall/")+4+len];

            MakeDirectory("system:config/uninstall", PERMIT_EVERYONE_READWRITE);

            len = StrCopy("system:config/uninstall/", buffer, COPY_ALL);
            len += StrCopy(Self->prvPackageName, buffer+len, COPY_ALL);
            len += StrCopy(".xml", buffer+len, COPY_ALL);
            ActionTags(MT_MoveFile, Self->prvUninstall, buffer);
         }
      }
      else {
         DPrintF("!Install:","Cannot produce an uninstall file as no package name was specified.");
         Action(MT_DeleteFile, Self->prvUninstall, NULL);
      }

      Action(AC_Free, Self->prvUninstall, NULL);
      Self->prvUninstall = NULL;
      Self->prvUninstallIndex = 0;
   }

   /*** Refresh the desktop if a program icon was installed ***/

   if (Self->prvRefreshDesktop) {
      OBJECTID desktopid;
      if (FastFindObject("desktop", NULL, &desktopid, 1, NULL) IS ERR_Okay) {
         struct acCustom custom;
         custom.Number = 0;
         custom.String = "RefreshIcons";
         ActionMsg(AC_Custom, desktopid, &custom);
      }
   }

   if (Self->prvXML) {
      Action(AC_Free, Self->prvXML, NULL);
      Self->prvXML = NULL;
   }

   if (Self->prvCompression) {
      Action(AC_Free, Self->prvCompression, NULL);
      Self->prvCompression = NULL;
   }
}

/*****************************************************************************
** Internal: TerminateInstall()
*/

void TerminateInstall(struct Install *Self)
{
   /*** Disable the installation buttons ***/

   if (Self->NextButtonID) ActionMsg(AC_Disable, Self->NextButtonID, NULL);
   if (Self->PrevButtonID) ActionMsg(AC_Disable, Self->PrevButtonID, NULL);
   if (Self->CancelButtonID) ActionMsg(AC_Disable, Self->CancelButtonID, NULL);

   /*** Reenable the navigation area ***/

   if (Self->NavigationID) ActionMsg(AC_Enable, Self->NavigationID, NULL);

   /*** Exit ***/

   if (Self->prvXML) {
      Action(AC_Free, Self->prvXML, NULL);
      Self->prvXML = NULL;
      Self->prvTags = NULL;
   }

   if (Self->prvCompression) {
      Action(AC_Free, Self->prvCompression, NULL);
      Self->prvCompression = NULL;
   }

   if (Self->prvUninstall) {
      Action(MT_DeleteFile, Self->prvUninstall, NULL);
      Action(AC_Free, Self->prvUninstall, NULL);
      Self->prvUninstall = NULL;
   }
}

/*****************************************************************************
** Internal: CancelInstall()
*/

void CancelInstall(struct Install *Self, WORD Message)
{
   /*** Disable the installation buttons ***/

   if (Self->NextButtonID) ActionMsg(AC_Disable, Self->NextButtonID, NULL);
   if (Self->PrevButtonID) ActionMsg(AC_Disable, Self->PrevButtonID, NULL);
   if (Self->CancelButtonID) ActionMsg(AC_Disable, Self->CancelButtonID, NULL);

   /*** Reenable the navigation area ***/

   if (Self->NavigationID) {
      if (Message) DisplayXML(Self, "<item><b><font colour=\"#202080\">Install Cancelled</font></b></item><item/><item>Installation cancelled.  You may now quit the installer, or choose another program to install.</item>");
      ActionMsg(AC_Enable, Self->NavigationID, NULL);
   }
   else if (Message) DisplayXML(Self, "<item><b><font colour=\"#202080\">Install Cancelled</font></b></item><item/><item>Installation cancelled.</item>");

   /*** Exit ***/

   if (Self->prvXML) {
      Action(AC_Free, Self->prvXML, NULL);
      Self->prvXML = NULL;
   }

   if (Self->prvCompression) {
      Action(AC_Free, Self->prvCompression, NULL);
      Self->prvCompression = NULL;
   }

   if (Self->prvUninstall) {
      Action(MT_DeleteFile, Self->prvUninstall, NULL);
      Action(AC_Free, Self->prvUninstall, NULL);
      Self->prvUninstall = NULL;
   }
}

/*****************************************************************************
** Internal: AddVariable()
*/

static ERROR AddVariable(struct Install *Self, STRING Name, STRING Value)
{
   struct XMLTag *tag, **xmllist;
   STRING str, replace;
   LONG namelen, valuelen, pos, i, count;

   if ((!Self) OR (!Name) OR (!Value)) return(ERR_Args);

   if (Self->prvTotalVariables >= MAX_VARIABLES) return(ERR_ArrayFull);

   for (namelen=0; Name[namelen]; namelen++);
   for (valuelen=0; Value[valuelen]; valuelen++);

   /*** Check if the variable is already listed ***/

   for (pos=0; pos < Self->prvTotalVariables; pos++) {
      if (StrCompare(Name, Self->prvVariables[pos].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) {
         FreeMemory(Self->prvVariables[pos].Name);
         Self->prvVariables[pos].Name  = NULL;
         Self->prvVariables[pos].Value = NULL;
         break;
      }
   }

   if (AllocMemory(namelen+1+valuelen+1, MEM_STRING|MEM_NOCLEAR, &str, NULL) IS ERR_Okay) {
      UBYTE variable[namelen+3];

      Self->prvVariables[pos].Name = str;
      while (*Name) *str++ = *Name++;
      *str++ = 0;

      Self->prvVariables[pos].Value = str;
      while (*Value) *str++ = *Value++;
      *str = 0;

      if (pos IS Self->prvTotalVariables) Self->prvTotalVariables++;

      /* Go through the XML tags and update anything that refers to the
      ** variable (identifiable by use of curly brackets).
      */

      variable[0] = '{';
      for (i=0; Self->prvVariables[pos].Name[i]; i++) variable[i+1] = Self->prvVariables[pos].Name[i];
      variable[i+1] = '}';
      variable[i+2] = 0;

      xmllist = Self->prvTags + Self->prvInstallIndex;
      for (count=0; count < Self->prvTotalTags; count++) {
          tag = xmllist[count];
          for (i=0; i < tag->TotalAttributes; i++) {
             if (tag->Attributes[i].Value) {
                /*** Replace all instances of the variable ***/
                if (StrReplace(tag->Attributes[i].Value, variable, Self->prvVariables[pos].Value, &replace, FALSE) IS ERR_Okay) {
                   /*** Update the XML structure ***/
                   ActionTags(MT_SetXMLAttribute, Self->prvXML, tag->Index, i, tag->Attributes[i].Name, replace);
                   tag = xmllist[count];
                   FreeMemory(replace);
                }
             }
          }
      }

      return(ERR_Okay);
   }
   else return(ERR_AllocMemory);
}

/*****************************************************************************
** Internal: CreateOption()
*/

static ERROR CreateOption(struct Install *Self, OBJECTID *ButtonID, STRING String, LONG OptionStatus)
{
   OBJECTPTR render, text, tickon, tickoff, onclick;
   LONG textheight;

   DPrintF("~CreateOption:","%s", String);

   /*** Create the option box ***/

   if (CreateObject(ID_RENDER, NF_SHARED, &render, ButtonID,
         FID_Container|TLONG, Self->ViewID,
         FID_XCoord|TLONG,    Self->prvCurrentXCoord,
         FID_YCoord|TLONG,    Self->prvCurrentYCoord,
         FID_XOffset|TLONG,   20,
         FID_Height|TLONG,    16,
         FID_Frame|TLONG,     1 + OptionStatus,
         FID_Region|TLONG,    TRUE,
         TAGEND) IS ERR_Okay) {

      /*** Create the text for the option box ***/

      CreateObject(ID_TEXT, NULL, &text, NULL,
         FID_Container|TLONG, *ButtonID,
         FID_String|TSTRING,  String,
         FID_YCoord|TLONG,    0,
         FID_Face|TSTRING,    glFontFace,
         FID_Colour|TSTRING,  "#000000",
         TAGEND);

      GetField(text, FID_TextHeight, FT_LONG, &textheight);
      SetFields(text, FID_XCoord|TLONG, textheight + 8, TAGEND);
      SetFields(render, FID_Height|TLONG, textheight, TAGEND);

      /*** Onclick frame management ***/

      CreateObject(ID_ONCLICK, NULL, &onclick, NULL,
         FID_Container|TLONG,    *ButtonID,
         FID_Flags|TLONG,        OCF_TOGGLEFRAME,
         FID_Frame|TLONG,        2 - OptionStatus,
         FID_ReleaseFrame|TLONG, 1 + OptionStatus,
         FID_Button|TLONG,       1,
         TAGEND);

      /*** Frame #1 graphics (option off) ***/

      CreateObject(ID_BOX, NULL, &tickoff, NULL,
         FID_Container|TLONG, *ButtonID,
         FID_Sunken|TLONG,    TRUE,
         FID_XCoord|TLONG,    0,
         FID_YCoord|TLONG,    0,
         FID_Width|TLONG,     textheight,
         FID_Height|TLONG,    textheight,
         FID_Colour|TSTRING,  "#f0f0f0",
         FID_Border|TSTRING,  "#000000",
         FID_Frame|TLONG,     1,
         TAGEND);

      /*** Frame #2 graphics (option on) ***/

      CreateObject(ID_BOX, NULL, &tickon, NULL,
         FID_Container|TLONG, *ButtonID,
         FID_Sunken|TLONG,    TRUE,
         FID_XCoord|TLONG,    0,
         FID_YCoord|TLONG,    0,
         FID_Width|TLONG,     textheight,
         FID_Height|TLONG,    textheight,
         FID_Colour|TSTRING,  "#00ff00",
         FID_Border|TSTRING,  "#000000",
         FID_Frame|TLONG,     2,
         TAGEND);

      Action(AC_Show, render, NULL);

      ReleaseObject(render);
   }
   else {
      StepBack();
      return(ERR_CreateObject);
   }

   Self->prvCurrentYCoord += textheight + 10;
   Self->prvIndent = textheight + 8;
   Self->prvBoxSize = textheight;

   StepBack();
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: DisplayXML()
*/

static OBJECTPTR DisplayXML(struct Install *Self, STRING Text)
{
   OBJECTPTR text;
   LONG len;

   if ((!Self) OR (!Text)) return(NULL);

   KillChildren(Self);

   ResetPage(Self);

   if (CreateObject(ID_TEXT, NULL, &text, NULL,
      FID_Container|TLONG,    Self->ViewID,
      FID_LeftMargin|TLONG,   3,
      FID_RightMargin|TLONG,  3,
      FID_TopMargin|TLONG,    3,
      FID_BottomMargin|TLONG, 3,
      FID_Face|TSTRING,       glFontFace,
      FID_Flags|TLONG,        TXF_WORDWRAP,
      FID_VScroll|TLONG,      Self->VSViewID,
      TAGEND) IS ERR_Okay) {

      for (len=0; Text[len]; len++);

      ActionTags(AC_DataChannel, text, Self->Head.UniqueID, DATA_XML, 1, Text, len, 1);

      return(text);
   }
   else return(NULL);
}

/*****************************************************************************
** Internal: FindTag()
*/

static struct XMLTag * FindTag(struct XMLTag *XMLTag, STRING Name)
{
   struct XMLTag *tag;

   while (XMLTag) {
      if (StrCompare(Name, XMLTag->Attributes[0].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) return(XMLTag);

      if (XMLTag->Child) {
         if ((tag = FindTag(XMLTag->Child, Name))) return(tag);
      }

      XMLTag = XMLTag->Next;
   }
   return(NULL);
}

/*****************************************************************************
** Internal: KillChildren()
*/

static void KillChildren(struct Install *Self)
{
   LONG count, i;

   if ((count = TotalChildren(Self->ViewID)) > 0) {
      struct ChildEntry list[count];

      DPrintF("~KillChildren()","Destroying children in the view area.");

      if (ListChildren(Self->ViewID, list, &count) IS ERR_Okay) {
         for (i=0; i < count; i++) ActionMsg(AC_Free, list[i].ObjectID, NULL);
      }
      if (Self->ViewID) ActionMsg(AC_Draw, Self->ViewID, NULL);

      StepBack();
   }

   Self->prvOutput = NULL;
}

/*****************************************************************************
** Internal: PreviousTag()
*/

static struct XMLTag * PreviousTag(struct XMLTag *XMLTag)
{
   struct XMLTag *tag;
   LONG i;

   if (!XMLTag) return(NULL);

   for (tag=XMLTag->Prev; tag; tag=tag->Prev) {
      for (i=0; glTags[i].Tag; i++) {
         if (StrCompare(glTags[i].Tag, tag->Attributes[0].Name, 0, STR_MATCHLENGTH) IS ERR_Okay) return(tag);
      }
   }

   return(XMLTag);
}
